#! /bin/sh

#
#	apache.sh - collect apache stats
#
#ident "@(#)cricket:PLANIX-1_0_3-V_1:apache.sh,v 1.1 2002/12/05 01:34:30 woods Exp"

# Usage:
#
# apache [URL]
#
# Where "URL" is the URL to a server's "server-status" page.  Make
# sure your server has granted access to this page to the machine you
# run this script on.  The magic "?auto" parameter is automatically
# appended to the end of any URL provided.
#
# This script munges the output of Apache's server-status function
# into a format suitable for Cricket's EXEC function.
#
# WARNING:  You _must_ have the "ExtendedStatus on" option enabled!
#
# the ouptput consists of:
#
# line	value	description
#
#  0	1594    Total Accesses
#  1	693     Total kBytes
#  2	.000503503      CPULoad
#  3	498507  Uptime
#  4	.00319755       ReqPerSec
#  5	1.42351 BytesPerSec
#  6	445.189 BytesPerReq
#  7	1       BusyServers
#  8	2       IdleServers
#  9	2       Processes waiting
# 10	0       Processes starting
# 11	0       Processes reading
# 12	1       Processes writing
# 13	0       Processes keepalive
# 14	0       Processes dns
# 15	0       Processes logging
# 16	0       Processes finishing
# 17	252     Processes open
# 18	255     Total Process slots
#


# Original version posted to cricket-users:
#
#Message-ID: <Pine.LNX.4.21.0004181010490.6485-100000@thundarr.nerdc.ufl.edu>
#Date: Tue, 18 Apr 2000 10:18:33 -0400 (EDT)
#From: James Moore <jm@ufl.edu>

# Select an appropriate tool to fetch the URL

#URL_GET="wget -q -O -"
#URL_GET="lynx -dump"
# NOTE: only recent NetBSD versions of ftp have '-o'
URL_GET="ftp -V -o - "

# pick your fastest, smallest, awk:
#
AWK=awk
#AWK=nawk
#AWK=gawk
#AWK=mawk

if [ $# -eq 1 ] ; then
	STATUS_URL=${1}"?auto"
else
	HOST=`hostname`
	STATUS_URL="http://$HOST/server-status?auto"
fi

# the 'tr' to strip CRs is only needed for url_get but is harmless for
# the rest....
# 
$URL_GET $STATUS_URL 2>/dev/null |
	$AWK '
		BEGIN {
			data_index = 0;
			score["waiting"] = 0;
			score["starting"] = 0;
			score["reading"] = 0;
			score["writing"] = 0;
			score["keepalive"] = 0;
			score["dns"] = 0;
			score["logging"] = 0;
			score["finishing"] = 0;
			score["open"] = 0;
			score["total"] = 0;
			scoreltr = "";
		}
		$1 ~ /[0-9][a-z][0-9]/ {
			next;
		}
		$1 == "Scoreboard:" {
			split($2, scorearr, "");
			for (i=1; i < length($2); i++) {
				scoreltr = scorearr[i];
				if (scoreltr == "_") {
					score["waiting"] += 1;
				} else if (scoreltr == "S") {
					score["starting"] += 1;
				} else if (scoreltr == "R") {
					score["reading"] += 1;
				} else if (scoreltr == "W") {
					score["writing"] += 1;
				} else if (scoreltr == "K") {
					score["keepalive"] += 1;
				} else if (scoreltr == "D") {
					score["dns"] += 1;
				} else if (scoreltr == "L") {
					score["logging"] += 1;
				} else if (scoreltr == "G") {
					score["finishing"] += 1;
				} else if (scoreltr == ".") {
					score["open"] += 1;
				}
				score["total"] += 1;
			}
			next;
		}
		/^[a-z A-Z]*:/ {
			descr = $0;
			sub(/: .*$/, "", descr);
			printf("%s\t%s\n", $NF, descr);
			next;
		}
		END {
			printf("%d\t%s\n", score["waiting"], "Processes waiting");
			printf("%d\t%s\n", score["starting"], "Processes starting");
			printf("%d\t%s\n", score["reading"], "Processes reading");
			printf("%d\t%s\n", score["writing"], "Processes writing");
			printf("%d\t%s\n", score["keepalive"], "Processes keepalive");
			printf("%d\t%s\n", score["dns"], "Processes dns");
			printf("%d\t%s\n", score["logging"], "Processes logging");
			printf("%d\t%s\n", score["finishing"], "Processes finishing");
			printf("%d\t%s\n", score["open"], "Processes open");
			printf("%d\t%s\n", score["total"], "Total Process slots");
		}
	'

exit 0
