#! /bin/sh

#
#	best-ups.sh - collect stats from a BEST FerrUPS via conserver
#
#ident "@(#)cricket:PLANIX-1_0_3-V_1:best-ups.sh,v 1.1 2002/12/05 01:34:30 woods Exp"

# Usage:
#
#	best-ups ups-console-name
#
#
# currently tuned for:
#
# Model:    FE3.1KVA
# Unit ID:  FE3.1K01494
# Serial #: FE3.1K01494
# Version:  8.03
#
# output is a line like this:
#
# 04260102410100010000000002450121000000140001690001054159980450003400000068080376
# 04/26 01:02:41 01 00 01 00 00 00 00 0245 0121 0000 001.4 000169 0001 054.1 59.98 0450 0034 0000 0068 0803 76
#
# index	pos	len	descr
# -----+-----+-------+------------------
# -1	hdr	n/a	<CR> <CR> <LF>
# 00	00	2	Month (Range: 01-12)
# 00	02	2	Day (Range: 01-31)
# 01	04	2	Hours (Range: 00-23)
# 01	06	2	Minutes (Range: 00-59)
# 01	08	2	Seconds (Range: 00-59)
# 02	10	2	System Mode: 00=Off, 01=Auto, 02=Line-Condition, 04=Batt-Power
# 03	12	2	reserved
# 04	14	2	Alarm Status: 00=Silenced, 01=Enabled
# 05	16	2	Inverter status: 00=off, 01=on-battery
# 06	18	2	Charger Status: 00=off, 01=Charging
# 07	20	2	Alarm Status Register #1 (HEX byte) (see below)
# 08	22	2	Alarm Status Register #2 (HEX byte) (see below)
# 09	24	4	Line AC Voltage In (Range: 0000-9999)
# 10	28	4	AC Voltage Out (Range: 0000-9999)
# 11	32	4	reserved
# 12	36	4	AC Output Current in tenths of Amps (Range: 000.0-999.9)
# 13	40	6	Volt-Amperes (VA) Out (Range: 000000-999999)
# 14	46	4	reserved (Battery Current in Amps?)
# 15	50	4	Battery Voltage in tenths of Volts (Range: 000.0-999.9)
# 16	54	4	(Line?) Frequency in centi-Hz (Range: 00.00-99.99)
# 17	58	4	Battery Runtime Minutes Remaining (Range: 0000-9999)
# 18	62	4	Temperature in degrees Celcius (Range: 0000-9999)
# 19	66	4	reserved?
# 20	70	2	Console Error Codes (see below)
# 21	72	2	Input Line Status (HEX byte) (see below)
# 22	74	4	Software version (x100) (Range: 00.00-99.99)
# 23	78	2	Checsum byte (2's compliment HEX sum, without carry)
# -1	trail	n/a	<CR> <CR> <LF>
#
# ASR#1 details:
#
# 0 - low battery
# 1 - near low battery
# 2 - high battery
# 3 - low runtime
# 4 - low AC output
# 5 - fan malfunction
# 6 - output overload
# 7 - external shutdown
#
# ASR#2 details:
#
# 0 - calibration lost
# 1 - user test
# 2 - replace battery
# 3-7 - reserved
#
# Console Error codes:
# 00 = No Error
# 01 = Unrecognised command
# 02 = Not implemented
# 03 = Number expected
# 04 = Bad/Missing keyword
# 05 = String expected
# 06 = Keyword or number expected
# 07 = Additional parameters expected
# 08 = Too many parameters
# 09 = Value out of range
# 0A = Bad password
# 0B = Password required
# 0C = Parameter not programmable
# 0D = Cannot change system mode
# 0E = Unrecognized error code
#
# Input Line status:
# 0 - Input frequency is bad
# 1 - Line glitch detected
# 2 - Input voltage too high or too low
# 3 - Line gone
# 4 - reserved
# 5 - reserved
# 6 - Inverter phase locked to line
# 7 - Inverter frequency locked to line
#
# 
# There are other parameters that are not included in the output of
# the "format" command, such as the alarm parameters and other
# temperature sensor values.  We'll add these on to the end of our
# output:
#
# 12 SinkTemp  34c        heatSinkTemp
# 14 XfmrTemp  35c        XfmrTemp
# 76 PFM Temp  34c        PfmTemp

# the conserver "console" client command
#
CONSOLE=/usr/local/bin/console

# pick your fastest, smallest, awk:
#
AWK=awk
#AWK=nawk
#AWK=gawk
#AWK=mawk

if [ $# -eq 1 ] ; then
	UPS_NAME=${1}
else
	echo "Usage: $argv0 ups-name" 1>&2
	exit 2
fi

(
	printf 'f\r'; sleep 1;
	printf "d 12\r";
	printf "d 14\r";
	printf "d 76\r"; sleep 1
) | $CONSOLE -f $UPS_NAME | tr '\015' '\012' | $AWK '
	BEGIN {
		separators[2] = "/";
		separators[4] = "\n";
		separators[6] = ":";
		separators[8] = ":";
		separators[10] = "\n";
		separators[12] = "\n";
		separators[14] = "\n";
		separators[16] = "\n";
		separators[18] = "\n";
		separators[20] = "\n";
		separators[22] = "\n";
		separators[24] = "\n";
		separators[28] = "\n";
		separators[32] = "\n";
		separators[36] = "\n";
		separators[39] = ".";
		separators[40] = "\n";
		separators[46] = "\n";
		separators[50] = "\n";
		separators[53] = ".";
		separators[54] = "\n";
		separators[56] = ".";
		separators[58] = "\n";
		separators[62] = "\n";
		separators[66] = "\n";
		separators[70] = "\n";
		separators[74] = "\n";
		separators[72] = "\n";
		separators[76] = ".";
		separators[78] = "\n";
		len = 0;
		i = 0;
	}
	length($0) > 68 {
		len = length($0);
		for (i = 0; i < len; i++) {
			if (separators[i] != "") {
				printf("%s", separators[i]);
			}
			printf("%s", substr($0, i+1, 1));
		}
		print "";
	}
	$1 == 12 || $1 == 14 || $1 == 76 {
		sub(/[cC]/, "", $NF);
		if ($(NF-1) ~ /.*[tT]emp/) {
			print $NF;
		} else {
			print "U";
		}
	}'
